@@ -51,6 +51,10 @@ pub struct Cursive {
 
     // Handle auto-refresh when no event is received.
     fps: Option<NonZeroU32>,
+
+    // List of callbacks to run on the backend.
+    // The current assumption is that we only add calls here during event processing.
+    pub(crate) backend_calls: Vec<Box<dyn FnOnce(&mut dyn backend::Backend)>>,
 }
 
 /// Identifies a screen in the cursive root.
@@ -95,6 +99,7 @@ impl Cursive {
             cb_sink,
             fps: None,
             user_data: Box::new(()),
+            backend_calls: Vec::new(),
         };
         cursive.reset_default_callbacks();
 
@@ -135,7 +140,7 @@ impl Cursive {
         if self.menubar.visible() {
             let printer = printer.focused(self.menubar.receive_events());
             printer.with_color(theme::ColorStyle::primary(), |printer| {
-                self.menubar.draw(&printer)
+                self.menubar.draw(printer)
             });
         }
 
@@ -219,6 +224,15 @@ impl Cursive {
         self.user_data().map(f)
     }
 
+    /// Sets the title for the terminal window.
+    ///
+    /// Note that not all backends support this.
+    pub fn set_window_title<S: Into<String>>(&mut self, title: S) {
+        let title = title.into();
+        self.backend_calls
+            .push(Box::new(move |backend| backend.set_title(title)));
+    }
+
     /// Show the debug console.
     ///
     /// Currently, this will show logs if [`logger::init()`](crate::logger::init()) was called.
@@ -285,7 +299,10 @@ impl Cursive {
 
     /// Selects the menubar.
     pub fn select_menubar(&mut self) {
-        self.menubar.take_focus(direction::Direction::none());
+        if let Ok(res) = self.menubar.take_focus(direction::Direction::none())
+        {
+            res.process(self);
+        }
     }
 
     /// Sets the menubar autohide feature.
@@ -306,18 +323,18 @@ impl Cursive {
     /// # use cursive_core::{Cursive, event};
     /// # use cursive_core::views::{Dialog};
     /// # use cursive_core::traits::*;
-    /// # use cursive_core::menu::*;
+    /// # use cursive_core::menu;
     /// #
     /// let mut siv = Cursive::new();
     ///
     /// siv.menubar()
     ///     .add_subtree(
     ///         "File",
-    ///         MenuTree::new()
+    ///         menu::Tree::new()
     ///             .leaf("New", |s| s.add_layer(Dialog::info("New file!")))
     ///             .subtree(
     ///                 "Recent",
-    ///                 MenuTree::new().with(|tree| {
+    ///                 menu::Tree::new().with(|tree| {
     ///                     for i in 1..100 {
     ///                         tree.add_leaf(format!("Item {}", i), |_| ())
     ///                     }
@@ -334,10 +351,10 @@ impl Cursive {
     ///     )
     ///     .add_subtree(
     ///         "Help",
-    ///         MenuTree::new()
+    ///         menu::Tree::new()
     ///             .subtree(
     ///                 "Help",
-    ///                 MenuTree::new()
+    ///                 menu::Tree::new()
     ///                     .leaf("General", |s| {
     ///                         s.add_layer(Dialog::info("Help message!"))
     ///                     })
@@ -544,16 +561,6 @@ impl Cursive {
         self.root.call_on_all(&view::Selector::Name(name), callback);
     }
 
-    /// Same as [`call_on_name`](Cursive::call_on_name).
-    #[deprecated(note = "`call_on_id` is being renamed to `call_on_name`")]
-    pub fn call_on_id<V, F, R>(&mut self, id: &str, callback: F) -> Option<R>
-    where
-        V: View,
-        F: FnOnce(&mut V) -> R,
-    {
-        self.call_on_name(id, callback)
-    }
-
     /// Convenient method to find a view wrapped in [`NamedView`].
     ///
     /// This looks for a `NamedView<V>` with the given name, and return
@@ -566,7 +573,7 @@ impl Cursive {
     /// # use cursive_core::Cursive;
     /// # use cursive_core::views::{TextView, ViewRef};
     /// # let mut siv = Cursive::new();
-    /// use cursive_core::traits::Identifiable;
+    /// use cursive_core::traits::Nameable;
     ///
     /// siv.add_layer(TextView::new("foo").with_name("id"));
     ///
@@ -582,7 +589,7 @@ impl Cursive {
     /// # use cursive_core::Cursive;
     /// # use cursive_core::views::{SelectView};
     /// # let mut siv = Cursive::new();
-    /// use cursive_core::traits::Identifiable;
+    /// use cursive_core::traits::Nameable;
     ///
     /// let select = SelectView::new().item("zero", 0u32).item("one", 1u32);
     /// siv.add_layer(select.with_name("select"));
@@ -606,33 +613,21 @@ impl Cursive {
         self.call_on_name(id, views::NamedView::<V>::get_mut)
     }
 
-    /// Same as [`find_name`](Cursive::find_name).
-    #[deprecated(note = "`find_id` is being renamed to `find_name`")]
-    pub fn find_id<V>(&mut self, id: &str) -> Option<views::ViewRef<V>>
-    where
-        V: View,
-    {
-        self.find_name(id)
-    }
-
     /// Moves the focus to the view identified by `name`.
     ///
     /// Convenient method to call `focus` with a [`view::Selector::Name`].
-    pub fn focus_name(&mut self, name: &str) -> Result<(), ViewNotFound> {
+    pub fn focus_name(
+        &mut self,
+        name: &str,
+    ) -> Result<EventResult, ViewNotFound> {
         self.focus(&view::Selector::Name(name))
     }
 
-    /// Same as [`focus_name`](Cursive::focus_name).
-    #[deprecated(note = "`focus_id` is being renamed to `focus_name`")]
-    pub fn focus_id(&mut self, id: &str) -> Result<(), ViewNotFound> {
-        self.focus(&view::Selector::Name(id))
-    }
-
     /// Moves the focus to the view identified by `sel`.
     pub fn focus(
         &mut self,
         sel: &view::Selector<'_>,
-    ) -> Result<(), ViewNotFound> {
+    ) -> Result<EventResult, ViewNotFound> {
         self.root.focus_view(sel)
     }
 
@@ -862,7 +857,7 @@ impl Cursive {
     ///
     /// Initializes a dummy backend for the event loop.
     pub fn run_dummy(&mut self) {
-        self.run_with(|| backend::Dummy::init())
+        self.run_with(backend::Dummy::init)
     }
 
     /// Returns a new runner on the given backend.
