@@ -1,8 +1,11 @@
+/// Event fired when the view is about to lose focus.
 use crate::{
     direction,
     event::{AnyCb, Event, EventResult, Key},
     rect::Rect,
-    view::{IntoBoxedView, Selector, SizeCache, View, ViewNotFound},
+    view::{
+        CannotFocus, IntoBoxedView, Selector, SizeCache, View, ViewNotFound,
+    },
     Printer, Vec2, With, XY,
 };
 use log::debug;
@@ -14,7 +17,7 @@ use std::ops::Deref;
 /// # Examples
 ///
 /// ```
-/// use cursive_core::traits::Boxable;
+/// use cursive_core::traits::Resizable;
 /// use cursive_core::views::{Button, LinearLayout, TextArea, TextView};
 ///
 /// let linear_layout = LinearLayout::horizontal()
@@ -39,7 +42,7 @@ struct Child {
 
     last_size: Vec2,
 
-    weight: usize,
+    _weight: usize,
 }
 
 impl Child {
@@ -114,9 +117,9 @@ impl<'a, T: Deref<Target = Child>, I: Iterator<Item = T>> Iterator
             self.offset += length;
 
             ChildItem {
+                child,
                 offset,
                 length,
-                child,
             }
         })
     }
@@ -144,20 +147,21 @@ impl LinearLayout {
         }
     }
 
-    /// Sets the weight of the given child.
+    /// Sets the weight of the given child. This weight is currently unused by the layout process.
     ///
     /// # Panics
     ///
     /// Panics if `i >= self.len()`.
     pub fn set_weight(&mut self, i: usize, weight: usize) {
-        self.children[i].weight = weight;
+        self.children[i]._weight = weight;
     }
 
-    /// Modifies the weight of the last child added.
+    /// Modifies the weight of the last child added. This weight is currently unused by the layout process.
     ///
     /// It is an error to call this before adding a child (and it will panic).
+    #[must_use]
     pub fn weight(mut self, weight: usize) -> Self {
-        self.children.last_mut().unwrap().weight = weight;
+        self.children.last_mut().unwrap()._weight = weight;
 
         self
     }
@@ -165,6 +169,7 @@ impl LinearLayout {
     /// Adds a child to the layout.
     ///
     /// Chainable variant.
+    #[must_use]
     pub fn child<V: IntoBoxedView + 'static>(self, view: V) -> Self {
         self.with(|s| s.add_child(view))
     }
@@ -175,7 +180,7 @@ impl LinearLayout {
             view: view.into_boxed_view(),
             required_size: Vec2::zero(),
             last_size: Vec2::zero(),
-            weight: 0,
+            _weight: 0,
         });
         self.invalidate();
     }
@@ -196,7 +201,7 @@ impl LinearLayout {
                 view: view.into_boxed_view(),
                 required_size: Vec2::zero(),
                 last_size: Vec2::zero(),
-                weight: 0,
+                _weight: 0,
             },
         );
         self.invalidate();
@@ -230,17 +235,24 @@ impl LinearLayout {
     pub fn set_focus_index(
         &mut self,
         index: usize,
-    ) -> Result<(), ViewNotFound> {
-        if self
-            .children
+    ) -> Result<EventResult, ViewNotFound> {
+        self.children
             .get_mut(index)
-            .map(|child| child.view.take_focus(direction::Direction::none()))
-            .unwrap_or(false)
-        {
+            .and_then(|child| {
+                child.view.take_focus(direction::Direction::none()).ok()
+            })
+            .map(|res| res.and(self.set_focus_unchecked(index)))
+            .ok_or(ViewNotFound)
+    }
+
+    fn set_focus_unchecked(&mut self, index: usize) -> EventResult {
+        if index != self.focus {
+            let result =
+                self.children[self.focus].view.on_event(Event::FocusLost);
             self.focus = index;
-            Ok(())
+            result
         } else {
-            Err(ViewNotFound)
+            EventResult::Consumed(None)
         }
     }
 
@@ -271,6 +283,12 @@ impl LinearLayout {
         self.children.get_mut(i).map(|child| &mut *child.view)
     }
 
+    /// Removes all children from this view.
+    pub fn clear(&mut self) {
+        self.invalidate();
+        self.children.clear();
+    }
+
     /// Removes a child.
     ///
     /// If `i` is within bounds, the removed child will be returned.
@@ -373,19 +391,17 @@ impl LinearLayout {
                 // We don't want that one.
                 self.iter_mut(true, rel)
                     .skip(1)
-                    .filter_map(|p| try_focus(p, source))
-                    .next()
+                    .find_map(|p| try_focus(p, source))
             })
-            .map_or(EventResult::Ignored, |i| {
-                self.focus = i;
-                EventResult::Consumed(None)
+            .map_or(EventResult::Ignored, |(i, res)| {
+                res.and(self.set_focus_unchecked(i))
             })
     }
 
     // Move the focus to the selected view if needed.
     //
     // Does nothing if the event is not a `MouseEvent`.
-    fn check_focus_grab(&mut self, event: &Event) {
+    fn check_focus_grab(&mut self, event: &Event) -> Option<EventResult> {
         if let Event::Mouse {
             offset,
             position,
@@ -393,11 +409,11 @@ impl LinearLayout {
         } = *event
         {
             if !event.grabs_focus() {
-                return;
+                return None;
             }
 
             let position = match position.checked_sub(offset) {
-                None => return,
+                None => return None,
                 Some(pos) => pos,
             };
 
@@ -419,27 +435,27 @@ impl LinearLayout {
                 // this will give us the allowed window for a click.
                 let child_size = item.child.last_size.get(self.orientation);
 
-                if item.offset + child_size > position {
-                    if item.child.view.take_focus(direction::Direction::none())
-                    {
-                        self.focus = i;
-                    }
-                    return;
+                if item.offset + child_size <= position {
+                    continue;
                 }
+
+                return item
+                    .child
+                    .view
+                    .take_focus(direction::Direction::none())
+                    .ok()
+                    .map(|res| res.and(self.set_focus_unchecked(i)));
             }
         }
+        None
     }
 }
 
 fn try_focus(
     (i, child): (usize, &mut Child),
     source: direction::Direction,
-) -> Option<usize> {
-    if child.view.take_focus(source) {
-        Some(i)
-    } else {
-        None
-    }
+) -> Option<(usize, EventResult)> {
+    child.view.take_focus(source).ok().map(|res| (i, res))
 }
 
 impl View for LinearLayout {
@@ -623,24 +639,24 @@ impl View for LinearLayout {
         compromise
     }
 
-    fn take_focus(&mut self, source: direction::Direction) -> bool {
+    fn take_focus(
+        &mut self,
+        source: direction::Direction,
+    ) -> Result<EventResult, CannotFocus> {
         // In what order will we iterate on the children?
         let rel = source.relative(self.orientation);
-        // We activate from_focus only if coming from the "sides".
-        let mut get_next_focus = || {
-            self.iter_mut(
-                rel.is_none(),
-                rel.unwrap_or(direction::Relative::Front),
-            )
-            .filter_map(|p| try_focus(p, source))
-            .next()
-        };
 
-        if let Some(i) = get_next_focus() {
-            self.focus = i;
-            true
+        // We activate from_focus only if coming from the "sides".
+        let focus_res = self
+            .iter_mut(rel.is_none(), rel.unwrap_or(direction::Relative::Front))
+            .find_map(|p| try_focus(p, source));
+
+        if let Some((next_focus, res)) = focus_res {
+            // No "FocusLost" here, since we didn't have focus before.
+            self.focus = next_focus;
+            Ok(res)
         } else {
-            false
+            Err(CannotFocus)
         }
     }
 
@@ -649,7 +665,9 @@ impl View for LinearLayout {
             return EventResult::Ignored;
         }
 
-        self.check_focus_grab(&event);
+        let res = self
+            .check_focus_grab(&event)
+            .unwrap_or(EventResult::Ignored);
 
         let result = {
             let mut iterator = ChildIterator::new(
@@ -661,7 +679,7 @@ impl View for LinearLayout {
             let offset = self.orientation.make_vec(item.offset, 0);
             item.child.view.on_event(event.relativized(offset))
         };
-        match result {
+        res.and(match result {
             EventResult::Ignored => match event {
                 Event::Shift(Key::Tab) if self.focus > 0 => {
                     self.move_focus(direction::Direction::back())
@@ -702,7 +720,7 @@ impl View for LinearLayout {
                 _ => EventResult::Ignored,
             },
             res => res,
-        }
+        })
     }
 
     fn call_on_any<'a>(
@@ -718,21 +736,20 @@ impl View for LinearLayout {
     fn focus_view(
         &mut self,
         selector: &Selector<'_>,
-    ) -> Result<(), ViewNotFound> {
+    ) -> Result<EventResult, ViewNotFound> {
         for (i, child) in self.children.iter_mut().enumerate() {
             if child.view.focus_view(selector).is_ok() {
-                self.focus = i;
-                return Ok(());
+                return Ok(self.set_focus_unchecked(i));
             }
         }
 
         Err(ViewNotFound)
     }
 
-    fn important_area(&self, _: Vec2) -> Rect {
+    fn important_area(&self, size: Vec2) -> Rect {
         if self.is_empty() {
             // Return dummy area if we are empty.
-            return Rect::from((0, 0));
+            return Rect::from_size(Vec2::zero(), size);
         }
 
         // Pick the focused item, with its offset
