@@ -1,12 +1,13 @@
-use crate::direction::Direction;
-use crate::event::{Event, EventResult, Key, MouseButton, MouseEvent};
-use crate::rect::Rect;
-use crate::theme::{ColorStyle, Effect};
-use crate::utils::lines::simple::{prefix, simple_prefix, LinesIterator, Row};
 #[allow(deprecated)]
-use crate::view::{ScrollBase, SizeCache, View};
-use crate::Vec2;
-use crate::{Printer, With, XY};
+use crate::{
+    direction::Direction,
+    event::{Event, EventResult, Key, MouseButton, MouseEvent},
+    rect::Rect,
+    theme::{ColorStyle, Effect},
+    utils::lines::simple::{prefix, simple_prefix, LinesIterator, Row},
+    view::{CannotFocus, ScrollBase, SizeCache, View},
+    Vec2, {Printer, With, XY},
+};
 use log::debug;
 use std::cmp::min;
 use unicode_segmentation::UnicodeSegmentation;
@@ -21,7 +22,7 @@ use unicode_width::UnicodeWidthStr;
 /// # Examples
 ///
 /// ```
-/// use cursive_core::traits::{Identifiable, Resizable};
+/// use cursive_core::traits::{Nameable, Resizable};
 /// use cursive_core::views::TextArea;
 ///
 /// let text_area = TextArea::new()
@@ -131,6 +132,7 @@ impl TextArea {
     /// Sets the content of the view.
     ///
     /// Chainable variant.
+    #[must_use]
     pub fn content<S: Into<String>>(self, content: S) -> Self {
         self.with(|s| s.set_content(content))
     }
@@ -145,6 +147,7 @@ impl TextArea {
     /// Disables this view.
     ///
     /// Chainable variant.
+    #[must_use]
     pub fn disabled(self) -> Self {
         self.with(Self::disable)
     }
@@ -157,6 +160,7 @@ impl TextArea {
     /// Re-enables this view.
     ///
     /// Chainable variant.
+    #[must_use]
     pub fn enabled(self) -> Self {
         self.with(Self::enable)
     }
@@ -421,7 +425,7 @@ impl TextArea {
             .map(|i| 1 + i + self.cursor);
         let last_row = last_byte
             .map_or(self.rows.len(), |last_byte| self.row_at(last_byte));
-        let last_byte = last_byte.unwrap_or_else(|| self.content.len());
+        let last_byte = last_byte.unwrap_or(self.content.len());
 
         debug!("Content: `{}` (len={})", self.content, self.content.len());
         debug!("start/end: {}/{}", first_byte, last_byte);
@@ -639,8 +643,13 @@ impl View for TextArea {
         EventResult::Consumed(None)
     }
 
-    fn take_focus(&mut self, _: Direction) -> bool {
+    fn take_focus(
+        &mut self,
+        _: Direction,
+    ) -> Result<EventResult, CannotFocus> {
         self.enabled
+            .then(|| EventResult::Consumed(None))
+            .ok_or(CannotFocus)
     }
 
     fn layout(&mut self, size: Vec2) {
